/******************************************************************************
 
 Stickman - Kinect Experiment v0.1
 
 Based on Max Rheiner's excellent work!
 http://code.google.com/p/simple-openni
 
 (c) 2012 Rolf van Gelder
 
 Release: 24-03-2012
 
 ******************************************************************************/

import SimpleOpenNI.*;

SimpleOpenNI  context;
boolean       autoCalib = true;
int           startTime = millis();
boolean       saved     = false;


/******************************************************************
 *
 * SETUP
 *
 ******************************************************************/
void setup()
{
  context = new SimpleOpenNI(this);

  // ENABLE DEPT MAP
  if (context.enableDepth() == false)
  {
    println("Can't open the depthMap, maybe the camera is not connected!"); 
    exit();
    return;
  }

  // ENABLE RGB MAP
  //  if (context.enableRGB() == false)
  //  {
  //    println("Can't open the rgbMap, maybe the camera is not connected or there is no rgbSensor!"); 
  //    exit();
  //    return;
  //  }  

  // ENABLE SKELETON TRACKING FOR ALL JOINTS
  context.enableUser(SimpleOpenNI.SKEL_PROFILE_ALL);

  stroke(0, 0, 255);
  strokeWeight(3);
  smooth();

  size(context.depthWidth(), context.depthHeight());
} // setup()


/******************************************************************
 *
 * DRAW
 *
 ******************************************************************/
void draw()
{
  background(200, 0, 0);

  // UPDATE THE CAM
  context.update();

  // image(context.depthImage(), 0, 0);
  // image(context.rgbImage(), 0, 0);

  // DRAW THE SKELETON (IF AVAILABLE)
  if (context.isTrackingSkeleton(1))
  {
    drawSkeleton(1);
    if (!saved && (millis()-startTime>20000))
    { // SAVE SCREENGRAB (ONCE, AFTER 20 SECS)
      save("capture"+millis()+".png");
      saved = true;
      println("SAVED.....................");
    }
  }
} // draw()


/******************************************************************
 *
 * DRAW THE SKELETON
 *
 ******************************************************************/
void drawSkeleton(int userId)
{
  // GET THE X AND Y POSITIONS ON THE JOINTS, CONVERTED TO THE CANVAS
  PVector neck = new PVector();
  context.getJointPositionSkeleton(userId, SimpleOpenNI.SKEL_NECK, neck);
  PVector xyNeck = new PVector();
  context.convertRealWorldToProjective(neck, xyNeck);

  PVector head = new PVector();
  context.getJointPositionSkeleton(userId, SimpleOpenNI.SKEL_HEAD, head);
  PVector xyHead = new PVector();
  context.convertRealWorldToProjective(head, xyHead);

  PVector leftShoulder = new PVector();
  context.getJointPositionSkeleton(userId, SimpleOpenNI.SKEL_LEFT_SHOULDER, leftShoulder);
  PVector xyLeftShoulder = new PVector();
  context.convertRealWorldToProjective(leftShoulder, xyLeftShoulder);

  PVector rightShoulder = new PVector();
  context.getJointPositionSkeleton(userId, SimpleOpenNI.SKEL_RIGHT_SHOULDER, rightShoulder);
  PVector xyRightShoulder = new PVector();
  context.convertRealWorldToProjective(rightShoulder, xyRightShoulder);

  PVector leftHip = new PVector();
  context.getJointPositionSkeleton(userId, SimpleOpenNI.SKEL_LEFT_HIP, leftHip);
  PVector xyLeftHip = new PVector();
  context.convertRealWorldToProjective(leftHip, xyLeftHip);

  PVector rightHip = new PVector();
  context.getJointPositionSkeleton(userId, SimpleOpenNI.SKEL_RIGHT_HIP, rightHip);
  PVector xyRightHip = new PVector();
  context.convertRealWorldToProjective(rightHip, xyRightHip);  

  PVector leftKnee = new PVector();
  context.getJointPositionSkeleton(userId, SimpleOpenNI.SKEL_LEFT_KNEE, leftKnee);
  PVector xyLeftKnee = new PVector();
  context.convertRealWorldToProjective(leftKnee, xyLeftKnee);

  PVector rightKnee = new PVector();
  context.getJointPositionSkeleton(userId, SimpleOpenNI.SKEL_RIGHT_KNEE, rightKnee);
  PVector xyRightKnee = new PVector();
  context.convertRealWorldToProjective(rightKnee, xyRightKnee);

  // MIDPOINT BETWEEN TOP OF THE HEAD AND NECK FOR DRAWING THE HEAD
  int midy = (int)(xyNeck.y + xyHead.y)/2;
  int midx = (int)(xyNeck.x + xyHead.x)/2;

  // DRAW HEAD
  pushStyle();
  fill(255, 0, 0);
  ellipse(midx, midy, 80, 80);
  popStyle();

  // CALCULATE THE GROIN POSITION FOR DRAWING THE LOWER BODY
  PVector xyGroin = intersect(xyLeftHip, xyRightKnee, xyRightHip, xyLeftKnee);

  // DRAW LOWER BODY
  beginShape();
  vertex(xyLeftHip.x, xyLeftHip.y);
  vertex(xyRightHip.x, xyRightHip.y);
  vertex(xyRightKnee.x, xyRightKnee.y);
  vertex(xyGroin.x, xyGroin.y);
  vertex(xyLeftKnee.x, xyLeftKnee.y);
  endShape(CLOSE);

  // context.drawLimb(userId, SimpleOpenNI.SKEL_HEAD, SimpleOpenNI.SKEL_NECK);

  context.drawLimb(userId, SimpleOpenNI.SKEL_NECK, SimpleOpenNI.SKEL_LEFT_SHOULDER);
  context.drawLimb(userId, SimpleOpenNI.SKEL_LEFT_SHOULDER, SimpleOpenNI.SKEL_LEFT_ELBOW);
  context.drawLimb(userId, SimpleOpenNI.SKEL_LEFT_ELBOW, SimpleOpenNI.SKEL_LEFT_HAND);

  context.drawLimb(userId, SimpleOpenNI.SKEL_NECK, SimpleOpenNI.SKEL_RIGHT_SHOULDER);
  context.drawLimb(userId, SimpleOpenNI.SKEL_RIGHT_SHOULDER, SimpleOpenNI.SKEL_RIGHT_ELBOW);
  context.drawLimb(userId, SimpleOpenNI.SKEL_RIGHT_ELBOW, SimpleOpenNI.SKEL_RIGHT_HAND);

  // context.drawLimb(userId, SimpleOpenNI.SKEL_LEFT_SHOULDER, SimpleOpenNI.SKEL_TORSO);
  // context.drawLimb(userId, SimpleOpenNI.SKEL_RIGHT_SHOULDER, SimpleOpenNI.SKEL_TORSO);

  // DRAW THE TORSO
  beginShape();
  vertex(xyLeftShoulder.x, xyLeftShoulder.y);
  vertex(xyRightShoulder.x, xyRightShoulder.y);
  vertex(xyRightHip.x, xyRightHip.y);
  vertex(xyLeftHip.x, xyLeftHip.y);
  endShape(CLOSE);

  // context.drawLimb(userId, SimpleOpenNI.SKEL_LEFT_SHOULDER, SimpleOpenNI.SKEL_LEFT_HIP);
  // context.drawLimb(userId, SimpleOpenNI.SKEL_RIGHT_SHOULDER, SimpleOpenNI.SKEL_RIGHT_HIP);
  // context.drawLimb(userId, SimpleOpenNI.SKEL_LEFT_HIP, SimpleOpenNI.SKEL_RIGHT_HIP);

  // context.drawLimb(userId, SimpleOpenNI.SKEL_LEFT_HIP, SimpleOpenNI.SKEL_RIGHT_KNEE);
  // context.drawLimb(userId, SimpleOpenNI.SKEL_RIGHT_HIP, SimpleOpenNI.SKEL_LEFT_KNEE);

  // context.drawLimb(userId, SimpleOpenNI.SKEL_TORSO, SimpleOpenNI.SKEL_LEFT_HIP);
  context.drawLimb(userId, SimpleOpenNI.SKEL_LEFT_HIP, SimpleOpenNI.SKEL_LEFT_KNEE);
  context.drawLimb(userId, SimpleOpenNI.SKEL_LEFT_KNEE, SimpleOpenNI.SKEL_LEFT_FOOT);

  // context.drawLimb(userId, SimpleOpenNI.SKEL_TORSO, SimpleOpenNI.SKEL_RIGHT_HIP);
  context.drawLimb(userId, SimpleOpenNI.SKEL_RIGHT_HIP, SimpleOpenNI.SKEL_RIGHT_KNEE);
  context.drawLimb(userId, SimpleOpenNI.SKEL_RIGHT_KNEE, SimpleOpenNI.SKEL_RIGHT_FOOT);
} // drawSkeleton()


/******************************************************************
 *
 * CALCULATE THE INTERSECTION BETWEEN TWO LINES (p1,p2) AND (p3,p4)
 *
 * http://zonalandeducation.com/mmts/intersections/intersectionOfTwoLines1/intersectionOfTwoLines1.html
 *
 ******************************************************************/
PVector intersect (PVector p1, PVector p2, PVector p3, PVector p4)
{
  PVector intersection = new PVector();

  // LINE 1: y = ax + b
  float a = (p1.y - p2.y) / (p1.x - p2.x);
  float b = p1.y - a * p1.x;
  // LINE 2: y = cx + d
  float c = (p3.y - p4.y) / (p3.x - p4.x);
  float d = p3.y - c * p3.x;

  intersection.x = (d-b)/(a-c);
  intersection.y = intersection.x * a + b;

  return intersection;
} // intersect()


/******************************************************************
 *
 * SimpleOpenNI EVENTS
 *
 ******************************************************************/
void onNewUser(int userId)
{
  println("onNewUser - userId: " + userId);
  println("  start pose detection");

  if (autoCalib)
    context.requestCalibrationSkeleton(userId, true);
  else    
    context.startPoseDetection("Psi", userId);
}

void onLostUser(int userId)
{
  println("onLostUser - userId: " + userId);
}

void onStartCalibration(int userId)
{
  println("onStartCalibration - userId: " + userId);
}

void onEndCalibration(int userId, boolean successfull)
{
  println("onEndCalibration - userId: " + userId + ", successfull: " + successfull);

  if (successfull) 
  { 
    println("  User calibrated !!!");
    context.startTrackingSkeleton(userId);
  } 
  else 
  { 
    println("  Failed to calibrate user !!!");
    println("  Start pose detection");
    context.startPoseDetection("Psi", userId);
  }
}

void onStartPose(String pose, int userId)
{
  println("onStartPose - userId: " + userId + ", pose: " + pose);
  println(" stop pose detection");

  context.stopPoseDetection(userId); 
  context.requestCalibrationSkeleton(userId, true);
}

void onEndPose(String pose, int userId)
{
  println("onEndPose - userId: " + userId + ", pose: " + pose);
}

